 
extends Spatial

# member variables here, example:
# var a=2
# var b="textvar"

var config = preload("res://config/game_config.gd")

const LINES = 4
const PASS_TIME = 0.3

const GAME_SPEED = 1.2
const MOTION_SPEED = 2.0
const ENDLESS_LOOKAHEAD = 10
const ENDLESS_OFFSET = 9
const KICK_SPEED = 1.5
const SLOWMO_SPEED = 0.5
const BALL_KICKED_OUT = 0.5
const BALL_KICKED_OUT_FIRST_OFS = 0.3
const BALL_PRE_KICKOUT_TIME = 0.7
const LOSEBALL_TIME = 3
const MAGNET_TIME = 8
const FINGER_TIME = 8

const CHAR_ANIM_SCALE = 1.0


const POWERUP_EXTRABALL = 0
const POWERUP_BULLETTIME = 1
const POWERUP_FREEZE = 2
const POWERUP_CHEER = 3
const POWERUP_SUPERBALL = 4

const BALL_KICKOUT_TIME = 0.8

const CONSUMABLE_EXTRABALL = 0
const CONSUMABLE_GOALKICK = 1

var char_order=["lechuga","capi","beto","loco"]

var slowmo_time=0.0
var glowball_time=0.0
var cheer_time=0.0
var finger_time=0.0
var invincible_time=0.0
var magnet_time=0.0

var game_speed_mult = 1.0
var level_speed_mult = 1.0
var high_detail=false

var ball_pre_kicked_out_time=-1
var ball_kicked_out_time=-1
var kicked_out_from=Vector3()
var kicked_out_pos=Vector3()
var ball_lost_time=-1
var ball_lost_to=-1
var ball_lost_offset=0

var light_node=null
var omnilight_node=null

var gamestate=null

var extraballs=1

var limit_x=0

var training_mode=false
var goalkeeped=false

class Player:

	var line=0
	var ofs=0
	var type=0
	var node=null
	var target_ofs=-1
	var active=false
	var fouled=0.0
	var finished=false


const OBSTACLE_ENEMY = 0
const OBSTACLE_MOVE1 = 1
const OBSTACLE_MOVE2 = 2
const OBSTACLE_MOVE3 = 3
const OBSTACLE_MOVE4 = 4
const OBSTACLE_MOVE5 = 5
const OBSTACLE_COSPEL = 6
const OBSTACLE_HEAD = 7
const OBSTACLE_EXTRAPLAYER = 8
const OBSTACLE_BARRIER = 9
const OBSTACLE_FINISH = 10
const OBSTACLE_FROZENGUY = 11
const OBSTACLE_FLYINGGUY = 12
const OBSTACLE_CONFUSEDGUY = 13
const OBSTACLE_SQUISHEDGUY = 14
const OBSTACLE_CONE = 15
const OBSTACLE_POWERUP = 16
const OBSTACLE_EXTRABALL = 17
const OBSTACLE_FINGER = 18
const OBSTACLE_MAGNET = 19

const ENEMY_RUNNER = 0
const ENEMY_SPRINTER = 1
const ENEMY_TACKLER = 2
const ENEMY_FOULER = 3
const ENEMY_ZIGZAGGER = 4
const ENEMY_STOPPED = 5
const ENEMY_GOALKEEPER = 6

const SPRINTER_SPEEDUP = 1.0
const TACKLER_SPEEDUP = 2.0
const ZIGZAG_SPEED = 2.0
const GOALKEEPER_FOLLOW_SPEED = 1.0
const GOALKEEPER_SPEED = 0.5
const FOULER_SPEEDUP = 0.2
const FOUL_TIME = 4.0
const HEADING_SPEED = 1.2


class Obstacle:

	var line=0
	var ofs=0
	var extra_ofs=0
	var node=null
	var type = 0
	var etype = 0
	var visited=[]
	var time_active=0.0
	var active=false
	var activated=false
	var processed=false
	var extra_zofs=0.0
	var custom={}
	
	
var players=[]
var enemies=[]
var advance=0.0
var enemy_start=0.0
var ball_holder=0
var passing_time=-1
var passing_from_pos=Vector3()
var ball_header=null
var heading_time=0
var activate_pu_time=-1

var pass_count=0
var coin_count=0
var coin_multiply=1

var kicking=false
var kicking_intercepted=false
var kicking_from
var kicking_to
var kick_progress

var goal_node
var beto_mode=false

#var pass_wait=-1
#var pass_target=-1
var endless=[]
var speed_changes=[]

var gameover=false

const SKELPATH = "skeleton/Skeleton"
const CHAR_SCALE = 0.08

var scroll_size=12
var scroll_background=[null,null]
var scroll_offset=0

var active=0

var ap_node_path = "AnimationPlayer" #NodePath("AnimationPlayer")
var base_node_path = "base" #NodePath("base")

var equipped_powerup = 0
var main_activated=false
var pu_icons=["extraball","slowmo","frozen","vuvuzela","superball"]

func _change_char_anim_speed(speed):

		for e in enemies:
			if (e.type==OBSTACLE_ENEMY):
				e.node.get_node("AnimationPlayer").set_speed(speed)
		for p in players:
			p.node.get_node("AnimationPlayer").set_speed(speed)

func _on_powerup_pressed():
	_on_powerup_activated(equipped_powerup,0,true)

func _on_powerup_update():

	if (not main_activated):
		return
		
	var pu = equipped_powerup
	
	var t = 0
	
	if (pu==POWERUP_EXTRABALL):

		#t=finger_time
		pass
	elif (pu==POWERUP_FREEZE):
		return
	elif (pu==POWERUP_BULLETTIME):
		t=slowmo_time
	elif (pu==POWERUP_SUPERBALL):
		t=glowball_time
	elif (pu==POWERUP_CHEER):
		t=cheer_time
	
	#get_node("powerup/time").set_value(t)
		

func _on_finger_activated():
		if (finger_time>0):
			return # cant doubletap
	
		var _fi_times=[4.0,10.0,16.0]
		finger_time=_fi_times[0]


func _on_powerup_activated(idx,level=0,main=false):

	var pu = idx

	print("pu: ",pu)
	var total=0.0
	
	if (pu==POWERUP_EXTRABALL):
		return # do nothing
	elif (pu==POWERUP_FREEZE):
		for e in enemies:
			if (e.active and e.type==OBSTACLE_ENEMY and e.etype!=ENEMY_FOULER):
				e.node.get_node("AnimationPlayer").stop()
				for x in e.node.get_meta("shared_mat_nodes"):
					x.set_material_override(frozen_mat)
				var n = e.node.get_node(SKELPATH+"/enemy-"+e.node.get_meta("variant")+"-hair")
				if (n):
					n.set_material_override(frozen_hair_mat)
					
				var n = preload("res://match/freezedust.xml").instance()
				n.set_emitting(true)
				e.node.add_child(n)					
				e.type=OBSTACLE_FROZENGUY
				e.etype=0

		get_node("powerup/anim2").play("snow")
		get_node("pu_sfx").play("freeze")

	elif (pu==POWERUP_BULLETTIME):
		if (slowmo_time>0):
			return # cant doubletap
		var _slowmo_times=[3.0,7.0,12.0]
		slowmo_time=_slowmo_times[level]
		get_node("powerup/anim").play("slowmo_in")
		_change_char_anim_speed(CHAR_ANIM_SCALE*SLOWMO_SPEED)
		total=slowmo_time
			

	elif (pu==POWERUP_SUPERBALL):
		if (glowball_time>0):
			return 
# cant doubletap
	
		get_node("ball").get_node("pelota").set_material_override(glowball_mat)
		get_node("ball").get_node("pelota/shine").set_emitting(true)
		var _gb_times=[10.0,10.0,16.0]
		glowball_time=_gb_times[level]
		total=glowball_time
		light_node.get_parent().remove_child(light_node)
		omnilight_node = preload("res://match/omnilight.xml").instance()
		omnilight_node.set_project_shadows(high_detail)
		get_node("ball/pelota").set_flag(GeometryInstance.FLAG_CAST_SHADOW,false)
		
		get_node("ball").add_child( omnilight_node )
		omnilight_node.set_translation(Vector3(0.5,0.5,0))
		get_node("pu_sfx").play("glowball")
		
		pass	
	elif (pu==POWERUP_CHEER):
		if (cheer_time>0):
			return # cant doubletap
	
		get_node("powerup/anim").play("vuvuzela_in")
		get_node("powerup/anim").queue("vuvuzela_cycle")
		var _ch_times=[4.0,10.0,16.0]
		cheer_time=_ch_times[level]
		for e in enemies:
			if (e.type==OBSTACLE_ENEMY and e.etype!=ENEMY_FOULER and e.active):				
				e.node.get_node("AnimationPlayer").play("2-run-stunned"+["1","2"][randi()%2]+"-cycle")
				e.type=OBSTACLE_CONFUSEDGUY
				e.etype=0
		
		get_node("pu_sfx").play("vuvuzela")
		pass	
	
	if (main):	
		main_activated=true
	
	
func _end_powerup(pu):

	if (pu==equipped_powerup and main_activated):
		main_activated=false
#		get_node("powerup/anim").play("deactivate")
	if (extraballs>0):
		get_node("powerup/type/upgrade "+str(extraballs)).hide()
	else:
		get_node("powerup/anim").play("deactivate")
		
	
	
	
	if (pu==POWERUP_FREEZE):
		pass
	elif (pu==POWERUP_BULLETTIME):
		get_node("powerup/anim").play("slowmo_out")
		_change_char_anim_speed(CHAR_ANIM_SCALE)
			

	elif (pu==POWERUP_SUPERBALL):
		get_node("ball").get_node("pelota").set_material_override(null)
		get_node("ball/pelota").set_flag(GeometryInstance.FLAG_CAST_SHADOW,true)
		get_node("ball").get_node("pelota/shine").set_emitting(false)
		get_node("ball").remove_child( omnilight_node )
		omnilight_node.free()
		omnilight_node=null
		get_node("camera").add_child(light_node)
		
	elif (pu==POWERUP_CHEER):
		get_node("powerup/anim").play("vuvuzela_out")
		pass	
		
	get_node("pu_sfx").stop_all()

func _char_enter_area(type,cam,char):
	if (char.has_node(ap_node_path)):
		char.get_node(ap_node_path).set_active(true)
	char.get_meta(base_node_path).active=true
	active+=1
	#	print("active: ",active)

func _char_exit_area(type,cam,char):
	if (char.has_node(ap_node_path)):
		char.get_node(ap_node_path).set_active(false)
	char.get_meta(base_node_path).active=false
	active-=1
#	print("active: ",active)
	

var evil_mats={}
var frozen_mat=null
var frozen_hair_mat=null
var glowball_mat=null
var powerup_mats=[null,null,null,null,null]
var finger_mat=null
var magnet_mat=null
var extraball_mat=null
var enemy_mat_names=["evil","fowlers","icecubes","rollings","tanneds","unibrownies","pulpo"]


func _add_prox_area(scene):

	var prox = ProximityArea.new()
	prox.set_aabb(AABB(Vector3(-1.5,-0.5,-0.5),Vector3(3,1,1)))
	prox.set_track_flag(ProximityArea.TRACK_CAMERAS,true)
	prox.connect("area_enter",self,"_char_enter_area",[scene])
	prox.connect("area_exit",self,"_char_exit_area",[scene])
	scene.add_child(prox)

func _add_shadow(scene):
	if (high_detail):
		return
	var ss = preload("res://match/shadow.xml").instance()
	var t = ss.get_transform()
	scene.add_child(ss)
	ss.set_transform( scene.get_transform().affine_inverse() * t )


func _gen_char(enemy,variant,line,flip=false):
	var p = preload("res://art/player.scn")
	var scene = p.instance()
	var mat_variant=null
	var begstr=""
	
	if (enemy and evil_mats.has(variant)):
		mat_variant=evil_mats[variant]
	
	if (enemy):
		begstr="enemy"
	else:
		begstr="char"
		
	var sbnodes=[]
	for x in scene.get_node(SKELPATH).get_children():
#		x.free()
#		continue
		var descr=x.get_name().split("-")
		if (descr.size()<2):
			continue
		if (descr[0]=="char" and descr[1]=="shared"):
			#shared body
			if (mat_variant):
				x.set_material_override(mat_variant)
				sbnodes.push_back(x)
			continue
		if (descr[0]!=begstr and not (descr[1]=="beto" and beto_mode)):
			x.free()
			continue
		if (descr[1]=="shared"):
			continue #anything shared goes on
		if (descr[1]!=variant and not (descr[1]=="beto" and beto_mode)):
			x.free()
		
	scene.set_scale(Vector3(1,1,1)*CHAR_SCALE)
	scene.set_meta("variant",variant)
	scene.set_meta("shared_mat_nodes",sbnodes)
	var ap = scene.get_node("AnimationPlayer")
	var rotn = false
	if (enemy):
		rotn = true
		ap.play("1-runslow-evil-cycle")
	else:
		rotn=false
		ap.play("2-run"+variant+"-cycle")
		var pain = preload("res://match/pain.xml").instance()
		pain.set_scale(Vector3(1,1,1)/CHAR_SCALE)
		pain.set_name("pain")
		scene.add_child(pain)
		pain.set_emitting(false)
	
	if (flip):
		rotn = not rotn	
	
	var r = scene.get_rotation()
	r.x=(-0.5+(line/(LINES-1.0))*0.02) # 2
	if (rotn):
		r.y=-PI*0.5
	else:
		r.y=PI*0.5	
	scene.set_rotation(r)
	ap.set_active(false)
	ap.set_default_blend_time(0.15)
	_add_prox_area(scene)

	if (not enemy):
		var un = preload("res://art/target.scn").instance()
		var t = Transform()
		t.basis = scene.get_transform().basis.inverse()
		t.basis[2]=t.basis[2]*0.7
		un.set_transform(t)
		if (variant.find("melli")!=0):
			un.get_node("cursor").set_flag(GeometryInstance.FLAG_VISIBLE,false)
#		un.set_name("cursor")

		scene.add_child(un)
		
		
				
	return scene
		
	

#var swipe_dist=-1.0
#var swipe_player=-1
#var swipe_begin = Vector2()


func _restart():

	advance=0
	ball_holder=0
	passing_time=-1
	gameover=false
	get_node("gameover").hide()
	for o in enemies:
		o.visited=[]
		o.activated=false
		o.time_active=0
		o.extra_ofs=0.0
		




func _find_player(pos,loose=false):

	if (loose):
		
		var pline = int(pos.y+LINES*0.5)
		var i=-1
		var closest_d=1e20
		var closest_idx=-1
		for p in players:
			i+=1
			if (p.line!=pline):
				continue
			var d = abs( (p.ofs+advance) - pos.x )
			if (d<closest_d):
				closest_d=d
				closest_idx=i

		return closest_idx	
	else:
		var i=0
		for p in players:
			var r = Rect2( p.ofs+advance-0.5,p.line - LINES*0.5,1,1 )
			if (r.has_point(pos)):
				return i
			i+=1
			
	return -1


func _find_enemy(pos,header=false,any=false):

	var i=0
	for o in enemies:
		if (not o.active):
			i+=1
			continue
		if ((header and o.type==OBSTACLE_HEAD)):
			
			var r = Rect2( o.ofs + o.extra_ofs + enemy_start - advance,o.line - LINES*0.5,1,1 )
			if (r.has_point(pos)):
				#print("active?? - ",o.active)
				return i
		i+=1
			
	return -1


func _find_char(pos,ret):

	ret["closest"]=-1
	ret["is_enemy"]=false


	var pline = int(pos.y+LINES*0.5)
	var i=-1
	var closest_d=1e20
	for p in players:
		i+=1
		if (p.line!=pline):		
			continue
		var d = abs( (p.ofs+advance) - pos.x )
		if (d<closest_d):
			closest_d=d
			ret.closest=i
			ret.is_enemy=false
			
	i=-1
	for o in enemies:
		i+=1
		if (not o.active or o.line!=pline):
			continue
		var d = abs((o.ofs + o.extra_ofs + enemy_start - advance) - pos.x)

		if (d<closest_d):
		
			if ((o.type==OBSTACLE_HEAD) or o.has_meta("finger")):
		
				closest_d=d
				ret.closest=i
				ret.is_enemy=true
				o.node.get_node("AnimationPlayer").play("headbutter")
				o.node.get_node("sfx").play("header")
				o.node.get_node("AnimationPlayer").queue("2-runcapi-cycle")
				var p = players[ball_holder]
				p.node.get_node("AnimationPlayer").play("kick1-"+p.node.get_meta("variant")+"-l")
				p.node.get_node("AnimationPlayer").queue("2-run"+p.node.get_meta("variant")+"-cycle")
				if (o.node.has_node("finger")):
					o.node.remove_and_delete_child( o.node.get_node("finger") )
					o.set_meta("finger",null)
				


func _input(ev):

	if (ev.type==InputEvent.MOUSE_BUTTON and ev.pressed):
	
		if (heading_time>0 or gameover):
			return
			
		var sp = ev.pos
		sp=get_scene().get_root().get_final_global_transform().affine_inverse().xform(sp)
		var p = get_node("camera/camera").project_ray_origin(sp)
		var n = get_node("camera/camera").project_ray_normal(sp)
		var inters = Plane(Vector3(0,1,0),0).intersects_ray(p,n)
		
		#var eidx = _find_enemy(Vector2(inters.x,inters.z),true) #find header
		
		var c = {}
		_find_char(Vector2(inters.x,inters.z),c)
		if (c.closest<0):
			return
		if (c.is_enemy):
			if (enemies[c.closest].type==OBSTACLE_HEAD):
					heading_time=1.0
					ball_header=enemies[c.closest]
			else:
				enemies[c.closest].node.get_node("AnimationPlayer").play("digglet")
				enemies[c.closest].node.get_node("sfx").play("finger")
				enemies[c.closest].type=OBSTACLE_SQUISHEDGUY
				enemies[c.closest].etype=0
		else:
			var pidx=c.closest
			if (pidx !=-1 and passing_time<0 and pidx!=ball_holder and players[pidx].fouled<=0.0 and not players[pidx].finished):
				var passing_to=pidx
				passing_time=PASS_TIME
				var p = players[ball_holder]
				var pd = players[passing_to]
				if (p.line < pd.line):
					p.node.get_node("AnimationPlayer").play("kick1-"+p.node.get_meta("variant")+"-l",0.05)
#					pd.node.get_node("AnimationPlayer").play("kick2-"+pd.node.get_meta("variant")+"-r")
				else:
					p.node.get_node("AnimationPlayer").play("kick1-"+p.node.get_meta("variant")+"-r",0.05)
#					pd.node.get_node("AnimationPlayer").play("kick2-"+pd.node.get_meta("variant")+"-l")
		
				p.node.get_node("AnimationPlayer").queue("2-run"+p.node.get_meta("variant")+"-cycle")
				
#				pd.node.get_node("AnimationPlayer").queue("2-run"+pd.node.get_meta("variant")+"-cycle")
				var v = get_node("sfx").play(["kick 1","kick 2","kick 3"][randi()%3])
				get_node("sfx").set_pitch_scale(v,1.0+randf()*0.3)
				passing_from_pos=get_node("ball").get_transform().origin
				ball_holder=passing_to
				pass_count+=1

				
func _process_zigzag(e,delta):
	var pz = e.custom.zig
	e.custom.zig-=delta
	var changed = false
	if (e.custom.zig<0):
		if (e.custom.follow):
			if (players[ball_holder].line == e.line):
				var changed = (e.custom.zig+delta >=0)
				e.custom.zig=-1				
				return changed
			elif (players[ball_holder].line < e.line):
				e.custom.zig=1.0
				e.custom.down=false
				changed=true
			elif (players[ball_holder].line > e.line):
				e.custom.zig=1.0
				e.custom.down=true
			changed=true
			
		else:
			if (e.custom.down):
				if (e.line==LINES-1):
					e.custom.down=false
			else:
				if (e.line==0):
					e.custom.down=true
					
			e.custom.zig=1.0
		
	var cds=1.0
	if (not e.custom.down):
		cds=-1.0
	if (e.custom.zig<0.5 and pz >=0.5):
		e.line+=cds

	var cs = cos((1.0-e.custom.zig)*PI)*0.5+0.5 #smoother zigzag
#				cs = e.custom.zig				
	if (e.custom.zig>=0.5):
			e.extra_zofs=(1.0-cs)*cds
	else:
			e.extra_zofs=-cs*cds
			
	return changed
								
																

func _process(delta):

#	print( "vertices: ",VS.get_render_info(VS.INFO_VERTICES_IN_FRAME) )
	game_speed_mult=level_speed_mult

	if (gameover):
		return


	if (invincible_time>0.0):
		invincible_time-=delta
		if (invincible_time<0.0):
			invincible_time=0
			

	if (activate_pu_time>0):
		activate_pu_time-=delta
		game_speed_mult*=0.1
		if (activate_pu_time<0):
			activate_pu_time=-1		
			_change_char_anim_speed(CHAR_ANIM_SCALE)
			_on_powerup_activated(equipped_powerup,0,true)
			if (equipped_powerup in [POWERUP_EXTRABALL, POWERUP_FREEZE]):
				_end_powerup(equipped_powerup)
			

	if (ball_pre_kicked_out_time>0):
		ball_pre_kicked_out_time-=delta
		var c = ball_pre_kicked_out_time / BALL_PRE_KICKOUT_TIME
		var hpos = kicked_out_from
		hpos.x-=BALL_KICKED_OUT_FIRST_OFS*(1.0-c)
		get_node("ball").set_translation(hpos)
		return


	if (ball_kicked_out_time>0):
		if (ball_kicked_out_time==BALL_KICKOUT_TIME):
			get_node("sfx").play("kick")

		ball_kicked_out_time-=delta
		var c = ball_kicked_out_time / BALL_KICKOUT_TIME

		
		var hpos = kicked_out_from
		hpos.x-=BALL_KICKED_OUT_FIRST_OFS
		hpos.y+=1.0		
		hpos = kicked_out_pos.linear_interpolate(hpos,c)
		hpos.y+=sin( c * PI )*0.5
		get_node("ball").set_translation(hpos)
		if (ball_kicked_out_time<0):
			_change_char_anim_speed(CHAR_ANIM_SCALE)
			for x in players:
				x.node.get_node("AnimationPlayer").play("lose-"+x.node.get_meta("variant"))
				x.node.get_node("AnimationPlayer").queue("lose-"+x.node.get_meta("variant")+"-cycle")
			for x in enemies:
				if (x.active and x.type==OBSTACLE_ENEMY):
					x.node.get_node("AnimationPlayer").play("win-capi-cycle")
		
			_game_over(training_mode)
			
		return
	
	if (ball_lost_time>0):
		game_speed_mult*=0.6
		ball_lost_time-=delta
		get_node("powerup/white").set_opacity(pow(ball_lost_time/LOSEBALL_TIME,4)*0.8)
		get_node("powerup/black").set_opacity(pow(1.0 - ball_lost_time/LOSEBALL_TIME,2))
		if (ball_lost_time<0):
			get_node("powerup/white").set_opacity(0)
			get_node("powerup/black").set_opacity(0)
			ball_lost_time=-1
			for x in players:
				x.node.get_node("AnimationPlayer").play("lose-"+x.node.get_meta("variant"))
				x.node.get_node("AnimationPlayer").queue("lose-"+x.node.get_meta("variant")+"-cycle")
			for x in enemies:
				if (x.type==OBSTACLE_ENEMY):
					x.node.free()
					
			_game_over(training_mode)
					
			return
			
					

			
		

	if (slowmo_time>0):
		game_speed_mult*=SLOWMO_SPEED
		slowmo_time-=delta
		if (slowmo_time<0):
			slowmo_time=-1
			_end_powerup(POWERUP_BULLETTIME)

	if (glowball_time>0):
		#game_speed_mult*=SLOWMO_SPEED
		glowball_time-=delta
		if (glowball_time<0):
			glowball_time=-1
			_end_powerup(POWERUP_SUPERBALL)

	if (cheer_time>0):
		cheer_time-=delta
		if (cheer_time<0):
			cheer_time=-1
			_end_powerup(POWERUP_CHEER)

	if (finger_time>0):
		finger_time-=delta
		if (finger_time<0):
			finger_time=-1
	
	if (magnet_time>0):
		magnet_time-=delta
		if (magnet_time<0):
			magnet_time=-1
			get_node("ball/pelota/magnet").set_emitting(false)


	_check_endless()
	_on_powerup_update()
	var zofs = LINES*0.5-0.7

	var idx=0
	var bh = Vector3()
	
	var players_active=false
	for p in players:
		var t = Vector3( p.ofs + advance, 0,p.line - zofs )
		if (p.finished):
			idx+=1
			continue
		players_active=true
		p.node.set_translation( t )
		if (idx==ball_holder):
			bh = t+Vector3(0.4,0,0)
			p.node.get_node("cursor/cursor").set_flag(GeometryInstance.FLAG_VISIBLE,true)
		else:
			p.node.get_node("cursor/cursor").set_flag(GeometryInstance.FLAG_VISIBLE,false)

			
		if (p.fouled>0.0):
			p.fouled-=delta
			if (p.fouled<=0.0):
				p.node.get_node("pain").set_emitting(false)

			
		if (p.target_ofs>=0):
			if (p.ofs < p.target_ofs):
				p.ofs+=delta*MOTION_SPEED
				if (p.ofs>p.target_ofs):
					p.ofs=p.target_ofs
					p.target_ofs=-1
					
			else:
				p.ofs-=delta*MOTION_SPEED
				if (p.ofs<p.target_ofs):
					p.ofs=p.target_ofs
					p.target_ofs=-1
			if (p.target_ofs==-1):
				p.node.get_node("AnimationPlayer").play("2-run"+p.node.get_meta("variant")+"-cycle")
				

					
					
		idx+=1

	if (kicking):
	
		var curve_blend=0.8
		var pos = kicking_from.linear_interpolate(kicking_to,pow(kick_progress,curve_blend))
		var slowdown = 1.0
		kick_progress+=delta*KICK_SPEED*slowdown
		var np = kicking_from.linear_interpolate(kicking_to,pow(kick_progress,curve_blend))
		
		if (kick_progress>=1):
			kick_progress=1
			_game_over(true)
			get_node("sfx").play("finalkick")
			goal_node.node.get_node("goal/AnimationPlayer").play("default")
			
			for x in players:
				x.node.get_node("AnimationPlayer").play("win-"+x.node.get_meta("variant"))
				x.node.get_node("AnimationPlayer").queue("win-"+x.node.get_meta("variant")+"-cycle")
				
			
			#kicking=false
		get_node("ball").set_translation(pos)
	

	if (not players_active):
		return
		
		
	var to_delete=[]
	var cam_x = get_node("camera").get_translation().x
	
	for e in enemies:
		e.node.set_translation( Vector3( e.ofs + e.extra_ofs + enemy_start - advance, 0,e.line + e.extra_zofs - zofs ) )
		if (!e.active):
			if (e.processed and e.node.get_translation().x < cam_x-10):
				to_delete.push_back(e)
			continue
		if (not e.processed):
			e.processed=true
		e.time_active+=delta*game_speed_mult	
		if (e.type==OBSTACLE_EXTRAPLAYER):
			if (e.node.get_global_transform().origin.x < get_node("camera").get_global_transform().origin.x):
				var p = Player.new()
				p.line=e.line
				p.ofs=e.node.get_transform().origin.x-advance
				p.node=e.node
				p.active=true
				players.push_back(p)
				var r = p.node.get_rotation()				
				r.y=-r.y
				p.node.set_rotation(r)
				
				var t = Transform()
				t.basis = e.node.get_transform().basis.inverse()
				t.basis[2]=t.basis[2]*0.7
				#t.basis[1]=-t.basis[1]
				
				p.node.get_node("cursor").set_transform(t)
						
				if (not high_detail):
					p.node.get_node("shadow").free()
					_add_shadow(p.node)
					p.node.get_node("shadow").set_translation(Vector3(0,01,0))
				
								

				p.node.get_node("AnimationPlayer").play("turnaround",0)
				p.node.get_node("AnimationPlayer").queue("2-run"+p.node.get_meta("variant")+"-cycle")
				enemies.erase(e)
		elif (e.type==OBSTACLE_FINISH or e.type==OBSTACLE_BARRIER):
				e.extra_ofs+=delta*GAME_SPEED*game_speed_mult
				limit_x = e.node.get_translation().x
		elif (e.type==OBSTACLE_FROZENGUY or e.type==OBSTACLE_SQUISHEDGUY):
				e.extra_ofs+=delta*GAME_SPEED*game_speed_mult
		elif (e.type==OBSTACLE_CONE):
				e.extra_ofs+=delta*GAME_SPEED*game_speed_mult
		elif (e.type==OBSTACLE_COSPEL and magnet_time>0):
			_process_zigzag(e,delta*ZIGZAG_SPEED*game_speed_mult)
		elif (e.type==OBSTACLE_ENEMY and e.etype!=ENEMY_FOULER and cheer_time>0):
				e.node.get_node("AnimationPlayer").play("2-run-stunned"+["1","2"][randi()%2]+"-cycle")
				e.type=OBSTACLE_CONFUSEDGUY
				e.etype=0
				continue

		
				
		if (!e.activated):
			if (finger_time>0 and e.type==OBSTACLE_ENEMY and e.etype!=ENEMY_FOULER):
				e.node.add_child( preload("res://match/finger.xml").instance() )
				e.set_meta("finger",true)
				
			if (e.etype==ENEMY_SPRINTER and e.time_active>0):
				e.node.get_node("AnimationPlayer").play("2-run-evil-cycle")
				get_node("sfx").play("sprint")
				e.activated=true
			if (e.etype==ENEMY_STOPPED and e.time_active>0):
				e.node.get_node("AnimationPlayer").play("defender-normal") #idle with cone
				e.activated=true
			if (e.etype==ENEMY_TACKLER and e.time_active>0):
				e.node.get_node("AnimationPlayer").play("slide-attack")
				e.node.get_node("sfx").play("tackle")
				var t = Transform()
				t.basis=e.node.get_transform().basis.inverse()
				t.origin= Vector3(0,0.5,0)
				var d = preload("res://match/dust.xml").instance()				
				d.set_transform(t)
				e.node.add_child(d)
				e.activated=true
			if (e.etype==ENEMY_FOULER and e.time_active>0):
				e.node.get_node("AnimationPlayer").play("fouler-run-cycle")
				e.activated=true
				e.custom=false
			if (e.etype==ENEMY_ZIGZAGGER and e.time_active>0):
				#e.node.get_node("AnimationPlayer").play("slide-attack")
				e.activated=true
			if (e.etype==ENEMY_GOALKEEPER and e.time_active>0):
				#e.node.get_node("AnimationPlayer").play("slide-attack")
				e.activated=true
		else:
					
			if (e.etype==ENEMY_SPRINTER):
				e.extra_ofs-=delta* SPRINTER_SPEEDUP*game_speed_mult
			elif (e.etype==ENEMY_STOPPED):
				e.extra_ofs+=delta*GAME_SPEED*game_speed_mult
			elif (e.etype==ENEMY_TACKLER):
				e.extra_ofs-=delta*TACKLER_SPEEDUP*game_speed_mult
			elif (e.etype==ENEMY_FOULER):
				if (e.custom):
					e.extra_ofs-=delta*FOULER_SPEEDUP*game_speed_mult		
				else:
					e.extra_ofs-=delta*TACKLER_SPEEDUP*game_speed_mult		
			elif (e.etype==ENEMY_GOALKEEPER):
				
				var dist_to_ball = e.node.get_translation().x - bh.x
				
				e.extra_ofs+=delta*GAME_SPEED*game_speed_mult
				if (e.custom.grabbed!=3):			
					var ms = 1.0
				
					if (e.custom.grabbed==0 and dist_to_ball<1.5):
						e.custom.grabbed=1
						
					if (e.custom.grabbed>0):
						ms*=2.0
				
					var changed = _process_zigzag(e,delta*GOALKEEPER_FOLLOW_SPEED*game_speed_mult*ms)
					
					if (changed): #e.custom.prevdown!=e.custom.down):
						e.custom.prevdown=e.custom.down
						if (e.custom.grabbed==0):
												
							if (e.custom.zig==-1):
								#standing							
								e.node.get_node("AnimationPlayer").play("defender-normal")
							elif (e.custom.down):
								
								e.node.get_node("AnimationPlayer").play("goalie-down-cycle")
							else:
								e.node.get_node("AnimationPlayer").play("goalie-up-cycle")
								
						elif (e.custom.grabbed==1):
							#final attempt! try to get the ball no matter what it takes
							if (e.custom.zig==-1):
								#standing							
								e.node.get_node("AnimationPlayer").play("goalie-catch-front")
							elif (e.custom.down):
								
								e.node.get_node("AnimationPlayer").play("goalie-catch-down")
								e.node.get_node("sfx").play("goalkeeper")
							else:
								e.node.get_node("AnimationPlayer").play("goalie-catch-up")
								e.node.get_node("sfx").play("goalkeeper")
							e.custom.grabbed=2
						elif (e.custom.grabbed==2):
							e.custom.grabbed=3
								
							
								
					
			elif (e.etype==ENEMY_ZIGZAGGER):
				e.extra_ofs-=delta*GOALKEEPER_SPEED*game_speed_mult		
				_process_zigzag(e,delta*ZIGZAG_SPEED*game_speed_mult)
				#var pz = e.custom.zig
				#e.custom.zig-=delta*ZIGZAG_SPEED*game_speed_mult
				#if (e.custom.zig<0):
				#	if (e.custom.follow):
				#		if (players[ball_holder].line == e.line):
				#			e.custom.zig=-1
				#			continue
				#		elif (players[ball_holder].line < e.line):
				#			e.custom.zig=1.0
				#			e.custom.down=false
				#		elif (players[ball_holder].line > e.line):
				#			e.custom.zig=1.0
				#			e.custom.down=true
				#		
				#	else:
				#		if (e.custom.down):
				#			if (e.line==LINES-1):
				#				e.custom.down=false
				#		else:
				#			if (e.line==0):
				#				e.custom.down=true
				#				
				#		e.custom.zig=1.0
					
				#var cds=1.0
				#if (not e.custom.down):
				#	cds=-1.0
				#if (e.custom.zig<0.5 and pz >=0.5):
				#	e.line+=cds
	
	
				#var cs = cos((1.0-e.custom.zig)*PI)*0.5+0.5 #smoother zigzag
#				#cs = e.custom.zig				
				#if (e.custom.zig>=0.5):
				#		e.extra_zofs=(1.0-cs)*cds
				#else:
				#		e.extra_zofs=-cs*cds
				
				
				
	for d in to_delete:
		d.node.queue_free()
		print("ERASED ONE!")
		enemies.erase(d)
		
	#check foulers and advances, since they cant be avoided
	for e in enemies:
	
		if (!e.active):
			continue
		if ((e.type!=OBSTACLE_ENEMY or e.etype!=ENEMY_FOULER) and (e.type<OBSTACLE_MOVE1 or e.type>OBSTACLE_MOVE5) and e.type!=OBSTACLE_FINISH):
			continue
		var r = Rect2( e.ofs + e.extra_ofs + enemy_start - advance - 0.3,e.line - LINES*0.5,0.5,1 )
		var idx=-1
		
		for p in players:
#			if (p.line!=e.line or p in e.visited):
			idx+=1
		
			if (p.line!=e.line):
				continue
			var t = Vector2( p.ofs + advance +0.5,p.line - zofs )
	
			if (!r.has_point(t)):
				continue		
		
		
			if (e.type==OBSTACLE_FINISH ):
				print("Should kick")
				if (players[ball_holder]==p and not kicking and goal_node and not goalkeeped):
					print("kick it!")
					kicking=true
					kicking_from=get_node("ball").get_translation()
					kicking_to=goal_node.node.get_translation()+Vector3(0.3,0,1.5)
					passing_time=-1
					ball_header=null
					kick_progress=0
					p.node.get_node("AnimationPlayer").play("kick1-"+p.node.get_meta("variant")+"-l")
					get_node("sfx").play("hitball")
				
				p.finished=true
				continue
		
			#if (players[ball_holder]==p):
			#	continue #let it lose?
			if (e.type==OBSTACLE_ENEMY):
				if (0):
					p.node.queue_free()
				else:
					p.node.get_parent().remove_child(p.node)
					var t = Transform() #p.node.get_transform().orthonormalized()
					t.origin.z=0.8/CHAR_SCALE							
					t.basis=Matrix3(Vector3(0,1,0),PI)
					p.node.set_transform(t)
					e.node.add_child(p.node)
					e.node.get_node("AnimationPlayer").play("fouler-push-cycle")
					p.node.get_node("AnimationPlayer").play("foulerdrag-"+p.node.get_meta("variant")+"-cycle")
					e.node.get_node("sfx").play("metal_hit")					
					t = Transform()
					t.basis=e.node.get_transform().basis.inverse()
					t.origin= Vector3(0,0.5,0)
					var d = preload("res://match/dust.xml").instance()				
					d.set_transform(t)
					e.node.add_child(d)
					e.activated=true
				var rehold=players[ball_holder]==p
				players.erase(p)
				e.custom=true
				if (rehold):
					passing_time=PASS_TIME
					ball_holder=0		
					var pd = players[ball_holder]
					pd.node.get_node("AnimationPlayer").play("kick2-"+pd.node.get_meta("variant")+"-r")
					pd.node.get_node("AnimationPlayer").queue("2-run"+pd.node.get_meta("variant")+"-cycle")
					get_node("sfx").play("kick")
					passing_from_pos=get_node("ball").get_transform().origin
				elif (ball_holder>idx):
					ball_holder-=1
			elif (e.type >=OBSTACLE_MOVE1 and e.type<=OBSTACLE_MOVE5 and not e.activated):

				
				p.target_ofs = e.type - OBSTACLE_MOVE1
				if (not e.node):
					continue
				e.activated=true
				if (p.target_ofs > p.ofs):
					p.node.get_node("AnimationPlayer").play("3-runfast-"+p.node.get_meta("variant")+"-cycle")
					get_node("sfx").play("movefwd")
				elif (p.target_ofs < p.ofs):
					p.node.get_node("AnimationPlayer").play("1-runslow-"+p.node.get_meta("variant")+"-cycle")
					get_node("sfx").play("moveback")
					
				if (e.node.has_node("AnimationPlayer")):
					e.node.get_node("AnimationPlayer").play("taken")
				#p.node.get_node("AnimationPlayer").queue("2-run"+p.node.get_meta("variant")+"-cycle")
		
			
												

	if (ball_lost_time>0):			
		#ball lost logic
		bh = ball_lost_to.node.get_transform().origin
		bh.x+=ball_lost_offset
		
	elif (passing_time>=0):
		#pass logic
		var c = 1.0 - passing_time / PASS_TIME
		
		bh = passing_from_pos.linear_interpolate(bh,c)
		bh.y = sin( c * PI )*0.5
		passing_time-=delta
		var pp = 0.7
		if (passing_time < PASS_TIME*pp and passing_time+delta > PASS_TIME*pp):
				var left = true
				var p = players[ball_holder]
				if (left):
					p.node.get_node("AnimationPlayer").play("kick2-"+p.node.get_meta("variant")+"-r")
				else:
					p.node.get_node("AnimationPlayer").play("kick2-"+p.node.get_meta("variant")+"-l")
				p.node.get_node("AnimationPlayer").queue("2-run"+p.node.get_meta("variant")+"-cycle")

		
		#if (passing_time<0):
			#ball_holder=passing_to
		#	passing_to=-1
	elif (ball_header!=null):
		#header logic
	
		var c = 1.0 - abs(heading_time - 0.5) * 2.0
		var hpos = ball_header.node.get_transform().origin
		var hup = ball_header.node.get_transform().basis[1]
		var hfr = ball_header.node.get_transform().basis[2]
		hpos+=hup.normalized()*0.8+hfr.normalized()*0.2
		bh = bh.linear_interpolate(hpos,c)
		bh.y+=sin( c * PI )*0.55
		heading_time-=delta*HEADING_SPEED
		if (heading_time<0):
			ball_header=null
			heading_time=-1	
		
	else:
		#enemy logic

		for e in enemies:
		
			if (!e.active):
				continue
			var r = Rect2( e.ofs + e.extra_ofs + enemy_start - advance - 0.3,e.line - LINES*0.5,0.5,1 )
			
			
			if (e.type==OBSTACLE_ENEMY):
				if (e.etype!=ENEMY_FOULER and r.has_point(Vector2(bh.x,bh.z))):
					if (invincible_time>0.0):
						pass #none
					elif (glowball_time>0.0):
						e.type=OBSTACLE_FLYINGGUY
						e.etype=0
						e.node.get_node("AnimationPlayer").play("flyaway1")
						e.node.get_node("sfx").play("flyaway")
						get_node("sfx").play("cospel")
						coin_count+=config.coin_value*coin_multiply
						get_node("powerup/coins/number").set_number( coin_count )

						continue
					elif (extraballs>0):
						#get_node("extraball/"+str(extraballs-1)+"/anim").play("disappear")
						get_node("powerup/anim").play("activate")
						var p=players[ball_holder]
						p.node.get_node("AnimationPlayer").play("dribble")
						p.node.get_node("AnimationPlayer").queue("1-runslow-"+p.node.get_meta("variant")+"-cycle")
						get_node("sfx").play("pu_activate")
						if (e.etype!=ENEMY_GOALKEEPER):
#							e.node.get_node("AnimationPlayer").play("loseball-l")
							e.node.get_node("AnimationPlayer").play("dribble-enemy1",0.05)
							e.node.get_node("AnimationPlayer").queue("1-runslow-evil-cycle")
						
						_change_char_anim_speed(0.1)
						activate_pu_time=0.5#1.0
						
						extraballs-=1
						invincible_time=1.5
						
						continue
					else:
						if (e.etype==ENEMY_GOALKEEPER):
							goalkeeped=true
						ball_lost_time=LOSEBALL_TIME
						var p=players[ball_holder]
						p.node.get_node("AnimationPlayer").play("loseball-l")
						ball_lost_to=e
						ball_lost_offset=get_node("ball").get_transform().origin.x - e.node.get_transform().origin.x
						_change_char_anim_speed(0.5)
						get_node("sfx").play("lost_ball")

						return
						
			if (e.type==OBSTACLE_COSPEL):
			
				var cr = Rect2( e.ofs + e.extra_ofs + enemy_start - advance - 0.3,e.line - LINES*0.5,0.8,1 )
				if (e.custom.taken==false and cr.has_point(Vector2(bh.x,bh.z))):
					e.node.get_node("AnimationPlayer").play("hide")				
					coin_count+=config.coin_value*coin_multiply
					get_node("powerup/coins/number").set_number( coin_count )
					if (not high_detail):
						e.node.get_node("shadow").set_flag(GeometryInstance.FLAG_VISIBLE,false)
					e.custom.taken=true
					e.node.get_node("sfx").play("cospel")

				continue

			if (e.type==OBSTACLE_EXTRABALL):						
				if (r.has_point(Vector2(bh.x,bh.z)) and extraballs<=0):
					e.node.get_node("AnimationPlayer").play("hide")				
					extraballs=1
				continue

			if (e.type==OBSTACLE_POWERUP):
			
				if (r.has_point(Vector2(bh.x,bh.z))):
					var n = get_node("powerup")
					n.get_node("type").set_texture(ResourceLoader.load("res://match/icons/icon_"+pu_icons[e.etype]+".png"))
					get_node("powerup/anim").play("activate")
					equipped_powerup=e.etype

				continue

			if (e.type==OBSTACLE_MAGNET):
			
				if (r.has_point(Vector2(bh.x,bh.z))):
					get_node("ball/pelota/magnet").set_emitting(true)
					magnet_time=MAGNET_TIME
					e.node.get_node("AnimationPlayer").play("hide")				

				continue

			if (e.type==OBSTACLE_FINGER):
			
				if (r.has_point(Vector2(bh.x,bh.z))):
					finger_time=FINGER_TIME
					e.node.get_node("AnimationPlayer").play("hide")				
					for e in enemies:
						if (e.type==OBSTACLE_ENEMY and e.etype!=ENEMY_FOULER and e.activated):
							e.node.add_child( preload("res://match/finger.xml").instance() )
							e.set_meta("finger",true)
						
				continue

			if (e.type==OBSTACLE_CONE):
				if (r.has_point(Vector2(bh.x,bh.z))):
				
					if (invincible_time<=0.0):
					
						ball_kicked_out_time=BALL_KICKOUT_TIME
						_change_char_anim_speed(0.1)
						kicked_out_from = get_node("ball").get_transform().origin
						get_node("sfx").play("lost_ball")
						e.node.get_node("sfx").play("cone")
	#						gameover=true
						kicked_out_pos=get_node("camera").get_transform().origin+Vector3(-5,4,0)		
						return
				
			
			for p in players:
				if (p.line!=e.line or p in e.visited):
					continue
				var t = Vector2( p.ofs + advance +0.5,p.line - zofs )
	
				if (!r.has_point(t)):
					continue		
				e.visited.push_back(p)


				if (e.type==OBSTACLE_CONE or e.type==OBSTACLE_SQUISHEDGUY):
						p.node.get_node("AnimationPlayer").play("evasion2-l")
						p.node.get_node("AnimationPlayer").queue("2-run"+p.node.get_meta("variant")+"-cycle")
				elif (e.type==OBSTACLE_ENEMY):

					if (e.etype==ENEMY_TACKLER):
						p.node.get_node("AnimationPlayer").play("slide-eva-"+p.node.get_meta("variant"))
						get_node("sfx").play("jump")
						p.node.get_node("AnimationPlayer").queue("2-run"+p.node.get_meta("variant")+"-cycle")
					elif (e.etype in [ENEMY_SPRINTER,ENEMY_STOPPED,ENEMY_GOALKEEPER]):
						p.node.get_node("AnimationPlayer").play("evasion1-l")
						p.node.get_node("AnimationPlayer").queue("2-run"+p.node.get_meta("variant")+"-cycle")
					else:
						
						var sd=["-l","-r"][randi()%2]
						var r = randi()
						var ri=r%2
						var pevade = ri==0
						print("ri: ",pevade," ",ri," ",r)
						if (pevade):
							p.node.get_node("AnimationPlayer").play("evasion1"+sd)
						p.node.get_node("AnimationPlayer").queue("2-run"+p.node.get_meta("variant")+"-cycle")
						if (!pevade):
							e.node.get_node("AnimationPlayer").play("evasion1"+sd)
						e.node.get_node("AnimationPlayer").queue("1-runslow-evil-cycle")
				elif (e.type==OBSTACLE_FROZENGUY or e.type==OBSTACLE_CONFUSEDGUY):
						p.node.get_node("AnimationPlayer").play("evasion1-l")
						p.node.get_node("AnimationPlayer").queue("2-run"+p.node.get_meta("variant")+"-cycle")
						
					
								
				

	if (not kicking):
		var ball = get_node("ball")
		var ball_mesh = ball.get_node("pelota")
		var ball_rot = ball_mesh.get_rotation()
		ball_rot.y+=delta*10
		ball_rot.z=0.05
		ball.set_translation( Vector3(bh.x,0,bh.z) )
		ball_mesh.set_rotation(ball_rot)
		var t = Vector3(0,0.2+bh.y,0)
		if (ball_lost_time<0 and passing_time<0):
			var plen=0.2
			var ptime=0.8
			var ofs = fmod(advance,ptime)/ptime
			if (ofs<0.5):			
				ofs*=2.0
				t.y+=sin(ofs*PI)*0.1
				ofs=pow(ofs,0.5)
				t.x=ofs*plen

			else:
				ofs=(ofs-0.5)*2.0
				ofs=pow(ofs,2.0)
				t.x=(1.0-ofs)*plen
			t.x-=0.2
			
		ball_mesh.set_translation(t)
		
	var nt = get_node("camera").get_translation()
	nt.x=advance
	if (limit_x and nt.x > limit_x-3):
		nt.x=limit_x-3
	get_node("camera").set_translation(nt)
	
	advance+=delta*GAME_SPEED*game_speed_mult
	get_node("powerup/meters/number").set_number(advance)

	
	
	if (advance > scroll_offset + scroll_size * 0.5):
		scroll_offset=advance-fmod(advance,scroll_size)+scroll_size
		var move = scroll_background[0]
		scroll_background[0]=scroll_background[1]
		scroll_background[1]=move
		move.set_translation(Vector3(scroll_offset,0,0))
		
		
		



func load_txt_level(path):

	var Obstacles = preload("res://editor/obstacles.gd")

	var f = File.new()
	var err = f.open(path,File.READ)
	

	var formation={}
	var tmp_obstacles={}
	
	
	var enemy_at=-1
	var l = f.get_line()
	var line=0
	while (!f.eof_reached()):
		for i in range(l.length()):
			var c = l[i]
			if (c=="P"):
				formation[Vector2(i,line)]=Obstacles.PLAYER
			elif (c.ord_at(0)>32):
			
				var key=Vector2(i,line)


				var ofs = i
				if (enemy_at==-1):
					enemy_at=ofs
				elif (ofs < enemy_at):
					enemy_at=ofs
				
				var type
				if (c=="3"):
					type=Obstacles.ADVANCE
				elif (c=="1"):
					type=Obstacles.RETREAT
				elif (c=="2"):
					type=Obstacles.MIDDLE
				elif (c=="4"):
					type=Obstacles.ADVANCE4
				elif (c=="5"):
					type=Obstacles.ADVANCE5
				elif (c=="S"):
					type=Obstacles.SPRINTER
				elif (c=="T"):
					type=Obstacles.TACKLER
				elif (c=="F"):
					type=Obstacles.FOULER
				elif (c=="Z"):
					type=Obstacles.ZIGZAGGER
				elif (c=="z"):
					type=Obstacles.ZIGZAGGER_REV
				elif (c=="K"):
					type=Obstacles.SHARK
				elif (c=="^"):
					type=Obstacles.CONE
				elif (c=="H"):
					type=Obstacles.HEAD
				elif (c=="X"):
					type=Obstacles.EXTRAPLAYER
				elif (c=="C"):
					type=Obstacles.COIN
				elif (c=="B"):
					type=Obstacles.BARRIER
				elif (c=="I"):
					type=Obstacles.FINISH
				else:
					type=Obstacles.ENEMY
					
				tmp_obstacles[key]=type
				
			
		l= f.get_line()
		line+=1

	var obstacles={}
	
	for o in tmp_obstacles:
		obstacles[Vector2(o.x-enemy_at,o.y)]=tmp_obstacles[o]
		
	var data = {}
	data["formation"]=formation
	data["level"]=obstacles

	load_level_data(data)				
	
		
		
func load_level(path):

	var f = File.new()
	var err = f.open(path,File.READ)
	if (err):
		return
		
	load_level_data(f.get_var())

func _sort_advances(e1,e2):
	return (e1.ofs < e2.ofs)
		
	
func load_song(song):

	print("SONG: ",song)
	if (song!=""):
		get_node("music").set_stream( ResourceLoader.load(song) )
#		get_node("music").play()

func _hexcol(c):
	var col = Color()
	col.r = ((c>>16)&0xff)/255.0
	col.g = ((c>>8)&0xff)/255.0
	col.b = ((c)&0xff)/255.0

	return col
	
func load_lighting(dif,spec,fogbeg,fogend,curve):

	get_node("camera/DirectionalLight").set_color(Light.COLOR_DIFFUSE,_hexcol(dif))
	get_node("camera/DirectionalLight").set_color(Light.COLOR_SPECULAR,_hexcol(spec))
	get_node("WorldEnvironment").get_environment().fx_set_param(Environment.FX_PARAM_FOG_BEGIN_COLOR,_hexcol(fogbeg))
	get_node("WorldEnvironment").get_environment().fx_set_param(Environment.FX_PARAM_FOG_END_COLOR,_hexcol(fogend))
	get_node("WorldEnvironment").get_environment().fx_set_param(Environment.FX_PARAM_FOG_ATTENUATION,curve)
	


func load_level_data(level,load_players=true,atofs=0.0):


#	level["field"]="garage"
	if (get_node("scroll").get_child_count()==0): #no background loaded
		
		if ("field" in level):
		
			var f = level.field
	
			if (f=="endless"):
				load_background("res://art/stage_circus/metegol_circo.scn")
				load_song("res://music/theme_endless.mpc")
				load_lighting(0xcda96c,0xdb7ecf,0xbb704a,0x691d4a,4.59)
			elif (f=="wooden"):
				load_background("res://art/stage_wood/metegol_madera.scn")
				load_song("res://music/theme_wooden.mpc")
				load_lighting(0xc6ab58,0xdbaba6,0xffcb65,0x8e4c14,4.59)
			elif (f=="lab"):
				load_background("res://art/stage_lab/metegol_lab.scn")
				load_song("res://music/theme_lab.mpc")
				load_lighting(0xc6c4bf,0xdbddd6,0x959bbf,0x9ea89a,6.53)
	#			get_node("WorldEnvironment").get_environment().fx_set_param(Environment.FX_PARAM_FOG_END_COLOR,Color(0,0,0,0))
			elif (f=="chinese"):
				load_background("res://art/stage_chinese/metegol_chino.scn")
				load_song("res://music/theme_chinese.mpc")
				load_lighting(0xc6c4bf,0xdbddd6,0xffdd9b,0xffffff,8.88)
			elif (f=="challenge"):
				load_background("res://art/stage_standard/metegol_estandar.scn")
				load_song("res://music/theme_challenge.mpc")
				load_lighting(0xc6c4bf,0xdbddd6,0xffdd9b,0xffffff,8.88)			
			elif (f=="garage"):
				load_background("res://art/stage_garage/metegol_taller.scn")
				load_song("res://music/theme_garage.mpc")
				load_lighting(0xc6c4bf,0xdbddd6,0x959bbf,0x9ea89a,6.53)
				get_node("camera/rain").set_emitting(true)
			else:
				load_background("res://art/stage_standard/metegol_estandar.scn")
				load_song("res://music/theme_endless.mpc")
		else:
				load_background("res://art/stage_circus/metegol_circo.scn")
				load_song("res://music/theme_endless.mpc")
				load_lighting(0xcda96c,0xdb7ecf,0xbb704a,0x691d4a,4.59)



	
	if (load_players && "formation" in level):
		for pos in level.formation:
			var p = Player.new()
			p.line=pos.y
			p.ofs=pos.x
			players.push_back(p)
	
	var Obstacles = preload("res://editor/obstacles.gd")
	var advances=[]
	var loaded_enemies = []
	for pos in level.level:
		print("enemyyy at ",pos)	
		var o = Obstacle.new()
		o.line=pos.y
		o.ofs=pos.x+atofs
		var lo=level.level[pos]
		if (lo==Obstacles.ADVANCE):
			o.type=OBSTACLE_MOVE3
			advances.push_back(o)
		elif (lo==Obstacles.RETREAT):
			o.type=OBSTACLE_MOVE1
			advances.push_back(o)
		elif (lo==Obstacles.MIDDLE):
			o.type=OBSTACLE_MOVE2
			advances.push_back(o)
		elif (lo==Obstacles.ADVANCE4):
			o.type=OBSTACLE_MOVE4
			advances.push_back(o)
		elif (lo==Obstacles.ADVANCE5):
			o.type=OBSTACLE_MOVE5
			advances.push_back(o)
		elif (lo==Obstacles.COIN):
			o.type=OBSTACLE_COSPEL
			o.custom={"down":(o.line<3),"zig":1.0,"follow":true,"taken":false}
			
		elif (lo==Obstacles.HEAD):
			o.type=OBSTACLE_HEAD
		elif (lo==Obstacles.CONE):
			o.type=OBSTACLE_CONE
		elif (lo==Obstacles.EXTRAPLAYER):
			o.type=OBSTACLE_EXTRAPLAYER
		elif (lo==Obstacles.BARRIER):
			o.type=OBSTACLE_BARRIER
		elif (lo==Obstacles.FINISH):
			#o.type=OBSTACLE_FINISH
			#cant any longer be assigned manually
			continue
		elif (lo==Obstacles.EXTRABALL):
			o.type=OBSTACLE_EXTRABALL
		elif (lo==Obstacles.END_OF_BLOCK):
			continue
		elif (lo==Obstacles.FINGER):
			o.type=OBSTACLE_FINGER			
		elif (lo==Obstacles.MAGNET):
			o.type=OBSTACLE_MAGNET			
		elif (lo>Obstacles.FINGER and lo<=Obstacles.VUVUZELA):
			o.type=OBSTACLE_POWERUP
			o.etype=lo-Obstacles.FINGER			
		else:
		    #enemy of many types
			o.type=OBSTACLE_ENEMY
			if (lo==Obstacles.SPRINTER):
				o.etype=ENEMY_SPRINTER
			elif (lo==Obstacles.FOULER):
				o.etype=ENEMY_FOULER
				o.custom=false
			elif (lo==Obstacles.TACKLER):
				o.etype=ENEMY_TACKLER
			elif (lo==Obstacles.STOPPER):
				o.etype=ENEMY_STOPPED
			elif (lo==Obstacles.ZIGZAGGER):
				o.etype=ENEMY_ZIGZAGGER
				o.custom={"down":(o.line<3),"zig":1.0,"follow":false}
				if (o.line==3):
					o.custom.down = not o.custom.down
			elif (lo==Obstacles.ZIGZAGGER_REV):
				o.etype=ENEMY_ZIGZAGGER
				o.custom={"down":(o.line==0),"zig":1.0,"follow":false}
				if (o.line==0):
					o.custom.down = not o.custom.down
			elif (lo==Obstacles.SHARK):
				o.etype=ENEMY_ZIGZAGGER			
				o.custom={"down":(o.line<3),"zig":1.0,"follow":true}
			else:
				o.etype=ENEMY_RUNNER
				
		
			#continue
		enemies.push_back(o)
		loaded_enemies.push_back(o)

	var zofs = LINES*0.5-0.7

	var enemy_at=10
	if (load_players):				
		var cchar=0
		var pline=[-1,-1,-1,-1]
								
		for p in players:
			p.node = _gen_char(false,char_order[int(p.line)%4],p.line)
#			p.node = _gen_char(false,char_order[cchar],p.line)
			p.node.set_meta("base",p)
			cchar=(cchar+1)%char_order.size()
			_add_shadow(p.node)		
			add_child(p.node)
			p.node.set_translation( Vector3( p.ofs, 0,p.line - zofs ) )
			if (pline[p.line]==-1 or p.ofs > pline[p.line]):
				pline[p.line]=p.ofs

		if (1): # sort the advances
			advances.sort_custom(self,"_sort_advances")
			for a in advances:
				var d=0
				
				if (a.type>=OBSTACLE_MOVE1 and a.type<=OBSTACLE_MOVE5):
					d=a.type-OBSTACLE_MOVE1
				elif (a.type==OBSTACLE_EXTRAPLAYER):
					d=2 # was it 2?
					
				if (d > pline[a.line]):
					pline[a.line]=d
					a.custom=1
				elif (d < pline[a.line]):
					pline[a.line]=d
					a.custom=-1
				else:
					a.custom=0

	var enemy_mat_idx=0
	var melli_index=0
	
	for e in loaded_enemies:
		#e.ofs-=enemy_at
		if (e.type==OBSTACLE_ENEMY):
			var matname=""
			if (e.etype==ENEMY_GOALKEEPER):
				matname="pulpo"
			elif (level.has("opponents")):
				matname=level.opponents
			else:
				matname=enemy_mat_names[(enemy_mat_idx+1)%(enemy_mat_names.size()-1)]
			
			enemy_mat_idx=(enemy_mat_idx+1)%enemy_mat_names.size()
#			e.node = _gen_char(true,"evil",e.line)
			e.node = _gen_char(true,matname,e.line)
			e.node.set_meta("base",e)			
			_add_shadow(e.node)
		elif (e.type==OBSTACLE_HEAD):
			melli_index+=1
			e.node = _gen_char(false,["melli1","melli2"][melli_index&1],e.line,true)
			e.node.get_node("AnimationPlayer").play("2-runcapi-cycle")
			e.node.set_meta("variant","capi")
			

			e.node.set_meta("base",e)
			_add_shadow(e.node)
		elif (e.type==OBSTACLE_BARRIER):
			
			e.node = preload("res://art/goal/goalscene.xml").instance()
			e.node.set_meta("base",e)
			goal_node=e
			if (1):
				var o = Obstacle.new()
				o.line=randi()%4
				o.ofs=e.ofs #+(randi()%3)*2
				o.type=OBSTACLE_ENEMY
				o.etype=ENEMY_GOALKEEPER
				o.custom={"down":(o.line<3),"zig":1.0,"follow":true,"prevdown":-1,"grabbed":0}
				loaded_enemies.push_back(o)
				enemies.push_back(o)
				
			for i in range(4):
				var o = Obstacle.new()
				o.line=i
				o.ofs=e.ofs+2
				o.type=OBSTACLE_FINISH
				loaded_enemies.push_back(o)
				enemies.push_back(o)
			e.ofs+=6
			
		elif (e.type==OBSTACLE_FINISH):
			e.node = Spatial.new() #preload("res://art/barrier/finish.xml").instance()
			e.node.set_meta("base",e)
		elif (e.type==OBSTACLE_EXTRAPLAYER):
			e.node = _gen_char(false,char_order[int(e.line)%4],e.line,true)
			e.node.set_meta("base",e)
			_add_shadow(e.node)
		elif (e.type>=OBSTACLE_MOVE1 and e.type<=OBSTACLE_MOVE5):
			if (e.custom>0):
				e.node = preload("res://match/advance.xml").instance()
				_add_shadow(e.node)
			elif (e.custom<0):
				e.node = preload("res://match/retreat.xml").instance()
				_add_shadow(e.node)
			else:
				e.node = preload("res://match/advance.xml").instance()
				_add_shadow(e.node)
#				e.node=Spatial.new()
			
		elif (e.type==OBSTACLE_COSPEL):
			e.node = preload("res://art/cospel/cospel.scn").instance()
			e.node.get_node("AnimationPlayer").set_active(false)
			var r = e.node.get_rotation()
			r.x=(-0.4+(e.line/(LINES-1.0))*0.2)
			e.node.set_rotation(r)
			_add_shadow(e.node)
		elif (e.type==OBSTACLE_POWERUP):
			e.node = preload("res://art/cospel/cospel.scn").instance()
			e.node.get_node("AnimationPlayer").set_active(false)
			e.node.get_node("coin").set_material_override(powerup_mats[e.etype])
			var r = e.node.get_rotation()
			r.x=(-0.4+(e.line/(LINES-1.0))*0.2)
			e.node.set_rotation(r)
			_add_shadow(e.node)
		elif (e.type==OBSTACLE_FINGER):
			e.node = preload("res://art/cospel/cospel.scn").instance()
			e.node.get_node("AnimationPlayer").set_active(false)
			e.node.get_node("coin").set_material_override(finger_mat)
			var r = e.node.get_rotation()
			r.x=(-0.4+(e.line/(LINES-1.0))*0.2)
			e.node.set_rotation(r)
			_add_shadow(e.node)
		elif (e.type==OBSTACLE_MAGNET):
			e.node = preload("res://art/cospel/cospel.scn").instance()
			e.node.get_node("AnimationPlayer").set_active(false)
			e.node.get_node("coin").set_material_override(magnet_mat)
			var r = e.node.get_rotation()
			r.x=(-0.4+(e.line/(LINES-1.0))*0.2)
			e.node.set_rotation(r)
			_add_shadow(e.node)
		elif (e.type==OBSTACLE_EXTRABALL):
			e.node = preload("res://art/cospel/cospel.scn").instance()
			e.node.get_node("AnimationPlayer").set_active(false)
			e.node.get_node("coin").set_material_override(extraball_mat)
			var r = e.node.get_rotation()
			r.x=(-0.4+(e.line/(LINES-1.0))*0.2)
			e.node.set_rotation(r)
			_add_shadow(e.node)
		elif (e.type==OBSTACLE_CONE):
			e.node = preload("res://art/cone.scn").instance()
			e.node.set_scale(Vector3(CHAR_SCALE,CHAR_SCALE,CHAR_SCALE))
			var r = e.node.get_rotation()
			r.x=(-0.5+(e.line/(LINES-1.0))*0.02) # 2
			r.y=-PI*0.5
			e.node.set_rotation(r)
			
			_add_shadow(e.node)
			
		_add_prox_area(e.node)
		
		e.node.set_meta("base",e)
		e.node.add_child( preload("res://match/obstacle_sfx.xml").instance() )


		add_child(e.node)
		e.node.set_translation( Vector3( e.ofs+4, 0.01,e.line - zofs) )
	if (load_players):
		enemy_start=enemy_at	

func load_background(path):

	if (get_node("scroll").get_child_count()>0):
		return

	if (0):
		scroll_background[0]=Spatial.new()
		scroll_background[1]=Spatial.new()
		get_node("scroll").add_child(scroll_background[0])
		get_node("scroll").add_child(scroll_background[1])
		return

		
	var bg = ResourceLoader.load(path)
	scroll_background[0]=bg.instance()
	scroll_background[1]=bg.instance()
	scroll_background[0].set_translation(Vector3(-scroll_size,0,0))
	get_node("scroll").add_child(scroll_background[0])
	get_node("scroll").add_child(scroll_background[1])
	


func _check_endless():

	if (endless.size()==0):
		return
	while(advance*2 + ENDLESS_LOOKAHEAD > endless[0].ofs):
		print("ADVANCE: ",(advance + ENDLESS_LOOKAHEAD))
		print("instance at ofs: ",endless[0].ofs)
		load_level_data(endless[0].piece,false,endless[0].ofs)
		speed_changes.push_back(Vector2(endless[0].ofs,endless[0].speed))
		endless.remove(0)
		if (endless.size()):
			print("NEXT AT: ",endless[0].ofs)
	
					
	while(speed_changes.size() and advance*2  > speed_changes[0].x):
		level_speed_mult=speed_changes[0].y
		speed_changes.remove(0)
			
func _sort_td(a,b):
	return a["ofs"] < b["ofs"]

func load_endless(path):
	
	var f = File.new()
	var err = f.open(path,File.READ)
	if (err):
		return

	randomize()
		
	var pieces=[]
	var section_len=0
	var section_speed=0
	var accum_dist=0
	

	load_level_data( { "formation":{ Vector2(0,0):0,Vector2(0,1):0,Vector2(0,2):0,Vector2(0,3):0}, "level":{} } )
	while(true):

		var l = f.get_line()
		var done = l=="" or f.eof_reached()

		if (done or l.begins_with("@")):
			#add sections
			if (pieces.size()):
				for i in range(section_len):
					var p = pieces[ randi() % pieces.size() ]
					var d = {}
					d["piece"]=p
					d["speed"]=section_speed
					var piece_len=0.0
					for pk in p.level:
						if (pk.x > piece_len):
							piece_len=pk.x
							
							
					d["ofs"]=accum_dist+ENDLESS_OFFSET
					print("piece: ",endless.size()," at ofs: ",d.ofs)
					accum_dist+=piece_len				
					if (1):
						var td=[]
						for pk in p.level:
							var p2={}
							var d2={}
							p2.level={}
							d2["ofs"]=d["ofs"]+pk.x						
							d2["speed"]=section_speed
							d2["piece"]=p2
							var k = Vector2(0,pk.y)
							p2.level[k]=p.level[pk]
							td.push_back(d2)
								
						td.sort_custom(self,"_sort_td")
						for x in td:
							endless.push_back(x)
								
					else:
						endless.push_back(d)
				pieces=[]
			section_len=0
		
		if (done):
			break

		if (l.begins_with("@")):
			#read a section
			pieces=[]
			l = l.substr(1,l.length())
			var vars = l.split(",")
			for x in vars:
				var v = x.split("=")
				if (v[0].strip_edges()=="len"):
					section_len=int(v[1])
				if (v[0].strip_edges()=="speed"):
					section_speed=float(v[1])
			
		elif (l.strip_edges()!=""):
			var f = File.new()
			var err = f.open("res://levels/endless/"+l+".lvl",File.READ)
			if (not err):
				var p = f.get_var()
				if (typeof(p)==TYPE_DICTIONARY):
					pieces.push_back(p)
			else:
				print("Error loading endless level : "+l)
				

			
	
			
func set_high_detail(enable):
	high_detail=enable
	if (high_detail):
		get_node("camera/DirectionalLight").set_project_shadows(true)
# disable to make faster in theory
#		get_node("WorldEnvironment").get_environment().set_enable_fx(Environment.FX_GLOW,true)
#	

func _pause_pressed(p):
	if (p):
		get_scene().set_pause(true)
		get_node("powerup/quit").show()
		get_node("powerup/pause_label").show()
		get_node("powerup/pause").set_normal_texture(preload("res://match/ui/play.png"))
		get_node("powerup/pause").set_pressed_texture(preload("res://match/ui/play_hl.png"))
	else:
		get_scene().set_pause(false)
		get_node("powerup/quit").hide()
		get_node("powerup/pause_label").hide()
		get_node("powerup/pause").set_normal_texture(preload("res://match/ui/pause.png"))
		get_node("powerup/pause").set_pressed_texture(preload("res://match/ui/pause_hl.png"))
	

func _game_over(win):

	if (win):
		load_song("res://music/theme_win.mpc")

	else:
		load_song("res://music/theme_lose.mpc")

	get_node("music").play()
	get_node("music").set_loop(false)
	
	get_node("pu_sfx").stop_all()

	gameover=true
	gamestate.result.win=win
	gamestate.result.coins=coin_count
	gamestate.result.passes=pass_count
	gamestate.result.meters=advance
	gamestate.result.goalkeeped=goalkeeped
	var endscreen = preload("res://screens/debriefing.xml").instance()
	var ps = gamestate.get_powerups()
	#if (ps[equipped_powerup]>=0):
	#	if (extraballs==0):
	#		ps[equipped_powerup]=0
	#	else:
	#		ps[equipped_powerup]=extraballs-1

	add_child(endscreen)
	get_node("powerup/quit").hide()
	get_node("powerup/pause_label").hide()
	get_node("powerup/pause").hide()
	gamestate.result.cospels=coin_count
	gamestate.result.passes=pass_count
	endscreen.activate(get_node("camera"))
	

func _give_up():
	for x in players:
		x.node.get_node("AnimationPlayer").play("lose-"+x.node.get_meta("variant"))
		x.node.get_node("AnimationPlayer").queue("lose-"+x.node.get_meta("variant")+"-cycle")
	for x in enemies:
		if (x.type==OBSTACLE_ENEMY):
			x.node.free()
	get_scene().set_pause(false)	
	_game_over(false)

func _exit_scene():
	get_scene().call_group(0, "global", "gameplay_end")


func _enter_scene():

	get_scene().call_group(0, "global", "gameplay_begin")

	gamestate = get_scene().get_root().get_node("gamestate")
	light_node = get_node("camera/DirectionalLight")
	beto_mode=gamestate.get_value("beto_mode")
	if (beto_mode):
		char_order=["beto","beto","beto","beto"]

	set_high_detail(gamestate.get_value("high_detail"))
	#set_high_detail(true)
	var bonus=false

	# Initalization here
	if Globals.has("current_level"):
		var cl = Globals.get("current_level")
		if cl.type == "level":
			print("loading level ", gamestate.levels[cl.level])
			var lpath = gamestate.levels[cl.level]
			if (lpath.find("bonus")!=-1):
				bonus=true
			load_level(lpath)
			#Globals.clear("current_level")
			gamestate.result.mode=gamestate.MODE_LEVEL
		elif cl.type == "endless":
			print("loading endless ", cl.level)
			load_endless(cl.level)
			coin_multiply=cl.bonus
			#Globals.clear("current_endless")
			gamestate.result.mode=gamestate.MODE_ENDLESS
			get_node("powerup/meters").show()
			get_node("powerup/meters/number").force_number(0)
			training_mode=true
		elif cl.type == "challenge":
			load_level_data(cl.level)
			gamestate.result.mode=gamestate.MODE_CHALLENGE
	elif (players.size()==0):
		load_txt_level("res://levels/level3.txt")

#	load_background("res://art/stage_circus/metegol_circo.scn")
#	load_background("res://art/stage_wood/metegol_madera.scn")
#	load_background("res://art/stage_lab/metegol_lab.scn")
#	load_background("res://art/stage_chinese/metegol_chino.scn")
#	load_background("res://art/stage_standard/metegol_estandar.scn")
	set_process_input(true)
	set_process(true)
	var ball = preload("res://art/ball.scn").instance()
	glowball_mat = ball.get_node("pelota").get_material_override()
	ball.get_node("pelota").set_material_override(null)
	ball.get_node("pelota/shine").set_emitting(false)
	ball.get_node("pelota/magnet").set_emitting(false)
#	ball.get_node("pelota").set_scale( Vector3(1,1,1)*CHAR_SCALE )
	ball.set_name("ball")
	_add_shadow(ball)
	add_child(ball)
	print("****FD: "+OS.get_data_dir())
	print("locale: "+OS.get_locale())

	equipped_powerup = gamestate.get_value("current_powerup")
	var powerups = gamestate.get_powerups()
	var pstatus = powerups[equipped_powerup]
	
#	pstatus=2
#	equipped_powerup = 4
	
	print("pstatus: ",pstatus);
	if (pstatus==-1):
		get_node("powerup/type").set_opacity(0.2)
		get_node("powerup/type/upgrade 1").hide()
		get_node("powerup/type/upgrade 2").hide()
		get_node("powerup/type/upgrade 3").hide()
		extraballs=0		
	elif (pstatus==0):
		get_node("powerup/type/upgrade 1").hide()
		get_node("powerup/type/upgrade 2").hide()
		get_node("powerup/type/upgrade 3").hide()
		extraballs=1
	elif (pstatus==1):
		get_node("powerup/type/upgrade 1").show()
		get_node("powerup/type/upgrade 2").hide()
		get_node("powerup/type/upgrade 3").hide()
		extraballs=2
	elif (pstatus==2):
		get_node("powerup/type/upgrade 1").show()
		get_node("powerup/type/upgrade 2").show()
		get_node("powerup/type/upgrade 3").hide()
		extraballs=3		
	elif (pstatus==3):
		get_node("powerup/type/upgrade 1").show()
		get_node("powerup/type/upgrade 2").show()
		get_node("powerup/type/upgrade 3").show()
		extraballs=4
		
	


	if 1:
		var n = get_node("powerup")
		n.get_node("type").set_texture(ResourceLoader.load("res://match/icons/icon_"+pu_icons[equipped_powerup]+".png"))
#		n.get_child("hilite").set_texture(ResourceLoader.load("res://match/icons/icon_"+pu_icons[i]+"_hl.png"))
		n.connect("pressed",self,"_on_powerup_pressed")
	if (bonus):
		get_node("powerup/anim").play("321bonus")
	else:
		get_node("powerup/anim").play("321go")
	
#	get_node("powerup/coins/number").force_number( gamestate.get_cospels() )
	get_node("powerup/coins/number").force_number( 0 )
	get_node("powerup/pause").connect("toggled",self,"_pause_pressed")
		
#	for x in range(extraballs):
#		get_node("extraball/"+str(x)).set_opacity(0.0)				
	
	get_node("powerup/quit").connect("pressed",self,"_give_up")
	
	print("powerups: : ",gamestate.get_powerups())


		


func _init():
	for x in enemy_mat_names:
		var m = FixedMaterial.new()
		m.set_texture(FixedMaterial.PARAM_DIFFUSE,ResourceLoader.load("res://art/variants/jugadorblender_"+x+".tex"))
		evil_mats[x]=m

	frozen_mat = FixedMaterial.new()
	frozen_mat.set_texture(FixedMaterial.PARAM_DIFFUSE,ResourceLoader.load("res://art/variants/jugadorblender_freezefx.tex"))
	frozen_mat.set_parameter(FixedMaterial.PARAM_SPECULAR,Color(1.0,1.0,1.0))
	frozen_mat.set_parameter(FixedMaterial.PARAM_SPECULAR_EXP,60.0)
	frozen_mat.set_texture(FixedMaterial.PARAM_DETAIL,preload("res://match/freezemap.png"))
	frozen_mat.set_texcoord_mode(FixedMaterial.PARAM_DETAIL,FixedMaterial.TEXCOORD_SPHERE)
	
	frozen_hair_mat = FixedMaterial.new()
	frozen_hair_mat.set_texture(FixedMaterial.PARAM_DIFFUSE,ResourceLoader.load("res://art/variants/hairfreeze.tex"))
	frozen_hair_mat.set_parameter(FixedMaterial.PARAM_SPECULAR,Color(1.0,1.0,1.0))
	frozen_hair_mat.set_parameter(FixedMaterial.PARAM_SPECULAR_EXP,60.0)
	frozen_hair_mat.set_texture(FixedMaterial.PARAM_DETAIL,preload("res://match/freezemap.png"))
	frozen_hair_mat.set_texcoord_mode(FixedMaterial.PARAM_DETAIL,FixedMaterial.TEXCOORD_SPHERE)
	
#	frozen_mat.set_texture(FixedMaterial.PARAM_GLOW,preload("res://match/freezemap.png"))
	for x in range(5):
		var m = FixedMaterial.new()
#		preload("res://art/powerups/powerup_extraball.tex"),

		var t=[
		preload("res://art/powerups/powerup_extraball.tex"),
		preload("res://art/powerups/powerup_slowmo.tex"),
		preload("res://art/powerups/powerup_freeze.tex"),
		preload("res://art/powerups/powerup_superball.tex"),
		preload("res://art/powerups/powerup_vuvuzela.tex")][x]
		m.set_texture(FixedMaterial.PARAM_DIFFUSE,t)
		m.set_parameter(FixedMaterial.PARAM_SPECULAR,Color(1.0,1.0,1.0))
		m.set_parameter(FixedMaterial.PARAM_SPECULAR_EXP,60.0)
		powerup_mats[x]=m
	if (1):
		var m = FixedMaterial.new()
		m.set_texture(FixedMaterial.PARAM_DIFFUSE,preload("res://art/powerups/powerup_finger.tex"))
		m.set_parameter(FixedMaterial.PARAM_SPECULAR,Color(1.0,1.0,1.0))
		m.set_parameter(FixedMaterial.PARAM_SPECULAR_EXP,60.0)
		finger_mat=m
		
		m = FixedMaterial.new()
		m.set_texture(FixedMaterial.PARAM_DIFFUSE,preload("res://art/powerups/powerup_magnet.tex"))
		m.set_parameter(FixedMaterial.PARAM_SPECULAR,Color(1.0,1.0,1.0))
		m.set_parameter(FixedMaterial.PARAM_SPECULAR_EXP,60.0)
		magnet_mat=m		
	
	
	extraball_mat = FixedMaterial.new()
	extraball_mat.set_texture(FixedMaterial.PARAM_DIFFUSE,preload("res://art/powerups/powerup_extraball.tex"))
	
	

