
extends TextureFrame

var game = null
var config = preload("res://config/game_config.gd")
var Obstacles = preload("res://editor/obstacles.gd")
# member variables here, example:
# var a=2
# var b="textvar"

var field_names=["challenge","garage","wooden","chinese","lab","endless"]
var enemy_names=["unibrownies","tanneds","icecubes","evil","fowlers","rollings"]
var obstacle_names=[Obstacles.ENEMY,Obstacles.SPRINTER,Obstacles.TACKLER,Obstacles.ZIGZAGGER,Obstacles.SHARK,Obstacles.STOPPER,Obstacles.CONE]

var current_shield=0
var current_table=0
var current_boost=0

var can_always_use_levels = false

var current_step = 1


func _table_pressed(idx):
	for i in range(6):
		var s = get_node("step 2/table_button "+str(i+1))
		s.set_pressed(i==idx)

	current_table=idx
	get_node("sfx").play("click")

func _shield_pressed(idx):
	for i in range(6):
		var s = get_node("step 2/shield_button "+str(i+1))
		s.set_pressed(i==idx)
	current_shield=idx
	get_node("sfx").play("click")


func _boost_pressed(idx):
	for i in range(3):
		var b = get_node("step 3/booster "+str(i+1))
		b.set_pressed(idx==i)
	current_boost=idx
	get_node("sfx").play("click")

func _update_page():

	get_node("top/back").set_disabled(current_step==1)
	get_node("top/next").set_disabled(current_step==3)
	
	for i in range(1,4):
		if (current_step==i):
			get_node("step "+str(i)).show()
		else:
			get_node("step "+str(i)).hide()
			
	get_node("top/step").set_frame(current_step-1)


func _on_next_pressed():
	current_step+=1
	if (current_step>3):
		current_step=3
	_update_page()
		


func _on_back_pressed():

	current_step-=1
	if (current_step<1):
		current_step=1
	_update_page()


func _on_send_and_test_pressed():
	var level = { "level": {}, "formation": [ Vector2(0, 0), Vector2(0, 1), Vector2(0, 2), Vector2(0, 3) ] }
	var edited = get_node("step 1/field/field_draw").stage
	var max_x = 0
	for key in edited.keys():
		if key.x > max_x:
			max_x = key.x
		var val = edited[key]	
		level.level[key] = obstacle_names[val]
	var bar_pos = Vector2(max_x+5, 0)
	level.level[bar_pos] = Obstacles.BARRIER
	level.field=field_names[current_table]
	level.opponents=enemy_names[current_shield]
	Globals.set("current_level", { "type": "challenge", "level": level, "mode": "local" } )
	get_scene().call_group(0, "global", "change_scene", "res://match/match.xml")


func _load_level():
	if !Globals.has("current_level"):
		return
	var cl = Globals.get("current_level")
	if cl.type != "challenge":
		return
	if cl.mode != "local":
		return
	var edited = {}
	for key in cl.level.level.keys():
		if cl.level.level[key] == Obstacles.BARRIER:
			continue
		edited[key] = obstacle_names.find(cl.level.level[key])

	var field = field_names.find(cl.level.field)
	for i in range(6):
		var s = get_node("step 2/table_button "+str(i+1))
		s.set_pressed(i==field)

	var opponents = enemy_names.find(cl.level.opponents)
	for i in range(6):
		var s = get_node("step 2/shield_button "+str(i+1))
		s.set_pressed(i==opponents)
	get_node("step 1/field/field_draw").stage = edited


func _ready():

	game = get_scene().get_root().get_node("gamestate")
	var features = game.get_value("features")
	if ("challenge_unlock" in features and features["challenge_unlock"]):
		can_always_use_levels=true

	var level = game.get_value("unlocked_level")
	#level=20
	for i in range(6):
		#set pressed con el que este seleccionado
		var s = get_node("step 2/shield_button "+str(i+1))
		var t = get_node("step 2/table_button "+str(i+1))
		var l = get_node("step 2/level_unlock "+str(i+1)+"/text")
		var ctable = config.get("challenge_scenario_"+str(i+1)+"_level")
		var cshield = config.get("challenge_opponent_"+str(i+1)+"_level")

		var disabled = not can_always_use_levels and level < ctable

		t.set_disabled(disabled)
		s.set_disabled(disabled)
		if (disabled):
			l.set_text(tr("GEAR_UP_LEVEL")+" "+str(ctable)+"+")
		else:
			l.set_text("")
		if (i==0):
			s.set_pressed(true)
			t.set_pressed(true)

		s.connect("pressed",self,"_shield_pressed",[i])
		t.connect("pressed",self,"_table_pressed",[i])

	for i in range(3):
		var b = get_node("step 3/booster "+str(i+1))
		if (i==0):
			b.set_pressed(true)
		b.connect("pressed",self,"_boost_pressed",[i])


	_load_level()

	_update_page()




func _on_home_pressed():
	get_scene().call_group(0, "global", "change_scene", "res://screens/main.xml")
